% Basic C, C++, and FORTRAN type mappings {{{
slirp_map_string("char*");
slirp_map_char("char");
slirp_map_char("signed char");
slirp_map_uchar("unsigned char");
slirp_map_short("short");
slirp_map_short("signed short");
slirp_map_ushort("unsigned short");
slirp_map_int("int");
slirp_map_int("signed int");
slirp_map_int("signed");
slirp_map_uint("unsigned");
slirp_map_uint("unsigned int");
slirp_map_long("long");
slirp_map_long("long int");
slirp_map_ulong("unsigned long");
slirp_map_ulong("unsigned long int");
slirp_map_float("float");
slirp_map_double("double");		% long double intentionally omitted
slirp_map_void(VOID);		% Handle funcs with void return types

slirp_map_ulong("size_t");
slirp_map_long("ptrdiff_t");

% The following provide size-guaranteed type mappings
define slirp_map_int16(ctype)	{ slirp_map_short(ctype);  }
define slirp_map_uint16(ctype)	{ slirp_map_ushort(ctype); }
define slirp_map_int32(ctype)	{ slirp_map_int(ctype);  }
define slirp_map_uint32(ctype)	{ slirp_map_uint(ctype); }
define slirp_map_int64(ctype)	{ slirp_map_long(ctype);  }
define slirp_map_uint64(ctype)	{ slirp_map_ulong(ctype); }


#ifeval SC.have_f2c
slirp_map_long("integer");
slirp_map_ulong("uinteger");
slirp_map_string("address");
slirp_map_short("shortint");
slirp_map_float("real");
slirp_map_double("doublereal");
slirp_map_scomplex("complex");
slirp_map_dcomplex("doublecomplex");
slirp_map_long("logical");
slirp_map_short("shortlogical");
slirp_map_char("logical1");
slirp_map_char("integer1");
slirp_map_double("E_f");
#endif

slirp_map_char("bool");
slirp_map_string("string");
%  }}}

%  IN/OUT argmaps for basic C (and some FORTRAN) types {{{

#argmap(out) short *OUTPUT
   $return ($1);
#end

#copy short *OUTPUT { unsigned short *OUTPUT, int *OUTPUT, unsigned int *OUTPUT, unsigned *OUTPUT, long *OUTPUT, unsigned long *OUTPUT, float *OUTPUT, double *OUTPUT }

#copy short *OUTPUT { short *OUT, unsigned short *OUT, int *OUT, unsigned int *OUT, unsigned *OUT, long *OUT, unsigned long *OUT, float *OUT, double *OUT}

#ifeval SC.have_f2c
#argmap(out) complex *OUTPUT
   SLang_push_complex($1.r, $1.i);
#end

#argmap(out) doublecomplex *OUTPUT
   SLang_push_complex($1.r, $1.i);
#end

#argmap(in, which=1) (char *, unsigned long FORTRAN_STRING_LEN)
   $2 = strlen($1);
#end
#endif

#argmap(out) size_t *OUTPUT
   SLang_push_size_t($1);
#end

#argmap(out) ptrdiff_t *OUTPUT
   SLang_push_ptrdiff_t($1);
#end
%  }}}

% Integer file descriptor proxy mapping % {{{
% This allows the S-Lang FD_Type to be passed to compiled routines
% expecting an integer file descriptor.  To achieve this we stub
% in a placeholder typemap entry, and define a proxy annotation.

slirp_map(SLFile_FD_Type, SLFile_FD_Type, "FD_Type", "SLANG_FILE_FD_TYPE",
      							TYPECL_POINTER);

#argmap(in, proxy=SLFile_FD_Type) int FD_PROXY
   if (-1 == SLfile_get_fd (proxy, &$1)) {
	SLang_verror(SLEI, "could not assign file descriptor proxy");
	return;
   }
#end
% }}}

% void* proxy mappings (Array_Type, Any_Type) % {{{

% As above, stub in a placeholder typemap entry for Any_Type
slirp_map(SLang_Array_Type, SLang_Array_Type, "array", "SLANG_ARRAY_TYPE",
      								TYPECL_POINTER);

variable vt = @SC.types[VOID];	% Create temporary void* type mapping,
vt.type += "*";			% just to evaluate proxy annotations
vt.ltype = SLang_Array_Type;
SC.types[vt.type] = vt;

#argmap(in, proxy=SLang_Array_Type) void* ARRAY_2_VOIDP
   $1 = proxy->data;
#end

slirp_map(SLang_Any_Type, SLang_Any_Type, "anytype", "SLANG_ANY_TYPE",
      								TYPECL_POINTER);

#argmap(in, proxy=SLang_Any_Type) void* ANY_2_VOIDP
   $1 = proxy;
#end

SC.types[vt.type] = NULL;	% discard void* mapping
% }}}

#argmap(in, proxy="char*") string % C++ string proxy {{{
   $1 = proxy;
#end
% }}}

% Mappings for NULL-terminated string arrays {{{
#typedef char** NT_STR_ARRAY
#typedef char** NT_STR_ARRAY_FREE

#retmap  NT_STR_ARRAY
   push_null_term_str_array($1, $funcname, 0);
#end

#retmap  NT_STR_ARRAY_FREE
   push_null_term_str_array($1, $funcname, 1);
#end
% }}}

provide("builtins");
